/*
 * Dynamic Particle System Library (DPS)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _FORCE_H_
#define _FORCE_H_

/*
 * Basicamente uma particula eh definida pela sua posicao, velocidade e massa. 
 * Estas tres unidades fisicas sao suficientes para modelar um sistema
 * newtoniano de massas, sujeitas a um numero determinado de forcas.
 *
 * Na nossa abordagem ao paradigma dos sistemas de particulas utilizamos
 * tres entidades distintas de interaccao mutua:
 *
 *       1. Grupos de Particulas
 *
 *       Basicamente sao conjuntos de particulas independentes, cada uma com o 
 *       seu conjunto de atributos (r, p, m).
 *
 *
 *       2. Grupos de Forcas
 *
 *       Sao definidos varios conjuntos de forcas, que sao aplicados a um 
 *       determinado conjunto de Particulas.
 *
 *
 *       3. Grupos de Objectos
 *
 *       Definimos tambem diversos tipos de dominios restritivos a aplicar 
 *       ao movimento das particulas, como por exemplo, locais onde as 
 *       particulas estao proibidas de entrar, ou objectos invisiveis nos 
 *       quais as particulas saltam apos uma colisao.
 *       Aqui definem-se tambem objectos de geracao e destruicao de particulas. 
 *       Resumindo, temos objectos geradores, destruidores de repulsao de 
 *       particulas.
 *
 *
 * A cada iteracao do sistema, sao aplicados um conjunto de Forcas e um
 * conjunto de Restricoes a um conjunto de Particulas. Esta accao ira 
 * recalcular novas posicoes para as particulas, baseando o seu movimento
 * num dado valor de delta tempo (dt).
 * Uma vez calculadas as novas posicoes das particulas soh eh preciso 
 * desenhar o conjunto das particulas.
 *
 * Algoritomo generico de funcionamento:
 *
 *    1. dpsIterate(P, F, R, dt).
 *    2. dpsDraw(P).
 *
 */

#include <stdio.h>
#include "types.h"
#include "particle.h"
#include "utility.h"
 
typedef struct _DPSforce_ {
    DPSenum            type;
    DPSfloat           dt;       //
    DPSvoid 	      *force;
} DPSforce; 		             // DPS force
 

typedef struct _DPSexplosion_{
    DPSvector3f position;	       // The center of the explosion
    DPSfloat    velocity;	       // Of shock wave
    DPSfloat    magnitude;	       // At unit radius
    DPSfloat    stdev;	       // Sharpness or width of shock wave
    DPSfloat    age;	       // How long it's been going on
    DPSfloat    epsilon;	       // Softening parameter
} DPSexplosion;


typedef struct _DPSfollow_ {
    DPSfloat magnitude;	// The grav of each particle
    DPSfloat epsilon;     	// Softening parameter
    DPSfloat max_radius;	// Only influence particles within max_radius
} DPSfollow; 


typedef struct _DPSgravitate_ {
    DPSfloat magnitude;	
    DPSfloat epsilon;  
    DPSfloat max_radius;
} DPSgravitate; 

typedef struct _DPSgravity_{
    DPSvector3f direction;
} DPSgravity;

typedef struct _DPSjet_{	       
    DPSvector3f    center;     	// Center of the fan
    DPSvector3f    acc;		// Acceleration vector domain
    DPSfloat       magnitude;	// Scales acceleration
    DPSfloat       epsilon;    	// Softening parameter
    DPSfloat       max_radius;	// Only influence particles within max_radius
} DPSjet;

typedef struct _DPSspeedLimit_{
	DPSfloat min_speed;		// Clamp speed to this minimum.
	DPSfloat max_speed;		// Clamp speed to this maximum.
} DPSspeedLimit;


typedef struct _DPStargetColor_{
	DPSvector3f   color;		// Color to shift towards AND alpha
	DPSfloat      scale;		// Amount to shift by (1 == all the way)
} DPStargetColor;

/*
typedef struct _DPStargetSize_{
	DPSvector3f size;		// Size to shift towards
	DPSvector3f scale;		// Amount to shift by per frame (1 == all the way)
}DPStargetSize;
*/
typedef struct _DPStargetVelocity_ {
	DPSvector3f velocity;	        // Velocity to shift towards
	DPSfloat    scale;		// Amount to shift by (1 == all the way)
}DPStargetVelocity;

typedef struct _DPSkillOld_ {
	DPSfloat    age_limit;    	// Exact age at which to kill particles.
	DPSboolean  kill_less_than;	// True to kill particles less than limit.
} DPSkillOld;


typedef struct _DPSdamping_{
    DPSvector3f  damping;	        // Damping constant applied to velocity
    DPSfloat     vlowSqr;		// Low and high cutoff velocities
    DPSfloat     vhighSqr;
} DPSdamping;

typedef struct _DPSorbitLine_ {
    DPSvector3f p, axis;            	// Endpoints of line to which particles are attracted
    DPSfloat magnitude;                	//Scales acceleration
    DPSfloat epsilon;                   // Softening parameter
    DPSfloat max_radius;              	// Only influence particles within max_radius
	
} DPSorbitLine;

typedef struct _DPSorbitPoint_ {
	DPSvector3f center;		// Point to which particles are attracted
	DPSfloat magnitude;	        // Scales acceleration
	DPSfloat epsilon;               // Softening parameter
	DPSfloat max_radius;	        // Only influence particles within max_radius
} DPSorbitPoint;

typedef struct _DPSvortex_{
    DPSvector3f center;		// Center of vortex
    DPSvector3f axis;		        // Axis around which vortex is applied
    DPSfloat magnitude;	        // Scale for rotation around axis
    DPSfloat epsilon;		// Softening parameter
    DPSfloat max_radius;            // Only influence particles within max_radius
} DPSvortex;


/*
 *  Function Signatures
 */


/*
 *  Memory allocation 
 */

void dpsApplyForce(DPSforce *force, GList** particles, DPSstate *state, DPSsets sets);

DPSforce* dpsNewGravity3f(DPSfloat x, DPSfloat y, DPSfloat z);

DPSforce* dpsNewFollow3f(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);

DPSforce* dpsNewGravitate3f(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);

DPSforce* dpsNewExplosion8f(DPSfloat x, DPSfloat y, DPSfloat z,
                            DPSfloat velocity, 
		            DPSfloat magnitude, 
		            DPSfloat stdev, 
		            DPSfloat epsilon,
		            DPSfloat age);

DPSforce* dpsNewExplosion9f(DPSfloat cx, DPSfloat cy, DPSfloat cz,
			    DPSfloat ax, DPSfloat ay, DPSfloat az,
			    DPSfloat magnitude, 
			    DPSfloat epsilon,
			    DPSfloat max_radius);

DPSforce* dpsNewSpeedLimit2f(DPSfloat min, DPSfloat max);

DPSforce* dpsNewTargetColor5f(DPSfloat c1, DPSfloat c2, DPSfloat c3, DPSfloat alpha, DPSfloat scale);

DPSforce* dpsNewTargetSize6f(DPSfloat sz0, DPSfloat sz1, DPSfloat sz2, DPSfloat sc0, DPSfloat sc1, DPSfloat sc2);

DPSforce* dpsNewTargetVelocity4f(DPSfloat v0, DPSfloat v1, DPSfloat v2, DPSfloat scale);

DPSforce* dpsNewKillOld2f(DPSfloat age_limit, DPSboolean  kill_less_than);

DPSforce* dpsNewdamping5f(DPSfloat d0, DPSfloat d1, DPSfloat d2, DPSfloat vlowSqr, DPSfloat vhighSqr);


DPSforce* dpsNewOrbitLine9f(DPSfloat p0,
			 DPSfloat p1, 
			 DPSfloat p2,
			 DPSfloat axis0,
			 DPSfloat axis1,
			 DPSfloat axis2, 
			 DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);

DPSforce* dpsNewOrbitPoint6f(DPSfloat c0, DPSfloat c1,DPSfloat c2, DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);


DPSforce* dpsNewVortex9f(DPSfloat c0,
			 DPSfloat c1, 
			 DPSfloat c2,
			 DPSfloat axis0,
			 DPSfloat axis1,
			 DPSfloat axis2, 
			 DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);
#endif
