/*
 * Dynamic Particle System Library (DPS)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _TYPES_H_
#define _TYPES_H_

#include <glib.h>
#include <GL/glut.h>


/*
 * DPS constants
 */


/*
 * DPS group arrays size (as posicoes apagadas nao sao reutilizadas)
 * por isso nao podemos criar mais que MAX grupos
 */
 
#define DPS_MAX_PARTICLE_GROUPS         2000
#define DPS_MAX_FORCE_GROUPS            2000
#define DPS_MAX_OBJECT_GROUPS           2000


/*
 * DPS draw bit mask options
 */

#define DPS_POINTS                      0x0001
#define DPS_MESHES                      0x0002
#define DPS_LINES                       0x0004
#define DPS_DROPS                       0x0008
#define DPS_FLOCKS                      0x0010
#define DPS_COLOR                       0x0040



/*
 * DPS truth values
 */

#define DPS_FALSE                       0
#define DPS_TRUE                        1
#define DPS_NULL                        0

/*
 * DPS Objects enum
 */

#define DPS_POINT                       0x01
#define DPS_LINE                        0x03
#define DPS_CUBE                        0x05
#define DPS_TRIANGLE                    0x07
#define DPS_RECTANGLE                   0x09
#define DPS_PLANE                       0x11
#define DPS_SPHERE                      0x13
#define DPS_CONE                        0x15
#define DPS_CYLINDER                    0x17
#define DPS_DISC                        0x19

/*
 * DPS ACTIONS enum
 */

#define DPS_AVOID                        0x01
#define DPS_COLLIDE                      0x03
#define DPS_KILL                         0x05
#define DPS_SOURCE                       0x07
#define DPS_JET                          0x09



/*
 * DPS forces enum
 */

#define DPS_EXPLOSION                     0x01
#define DPS_GRAVITY                       0x03
#define DPS_FOLLOW                        0x05
#define DPS_GRAVITATE                     0x07
#define DPS_JET                           0x09
#define DPS_SPEEDLIMIT                    0x11
#define DPS_TARGETCOLOR                   0x13
#define DPS_TARGETSIZE                   0x15
#define DPS_TARGETVELOCITY                0x17
#define DPS_RANDOMACCEL                   0x19
#define DPS_RANDOMDISPLACE                0x21
#define DPS_RANDOMVELOCITY                0x23
#define DPS_KILLOLD                       0x25
#define DPS_DAMPING                       0x27
#define DPS_ORBITLINE                     0x29
#define DPS_ORBITPOINT                    0x31
#define DPS_VORTEX                        0x33




/*
 * Math stuff
 */

#define DPS_PRECISION_ACOSINE         200   /* Bigger means more precision */
#define DPS_PRECISION_COSINE          200   /* Bigger means more precision */
#define DPS_PRECISION_SINE            200  /* Bigger means more precision */
#define DPS_EPSILON 	              1e-3f 
#define DPS_SQRT_2_PI 		      2.506628274631000502415765284811045253006
#define DPS_ONE_OVER_SQRT_2_PI 	      (1. / DPS_SQRT_2_PI)
#define DPS_MAXFLOAT                  1.0e16f /* Actually this must be < sqrt(MAXFLOAT) since we store this value squared.*/
#define DPS_ONE_OVER_SIGMA_EXP        (1.0f/0.7975f)


/*
 * Major DPS types defined
 */
typedef float           DPSfloat;
typedef DPSfloat        DPSvector3f[3];   // float vector with 3 positions
typedef DPSfloat        DPSvector4f[4];   // float vector with 4 positions
typedef unsigned int    DPSenum;          // enumeration type
typedef unsigned char   DPSboolean;       // boolean type
typedef void            DPSvoid;          // void type
typedef int	        DPSsizei;         // tamanho de ranges
typedef unsigned long   DPSulong;         // positive long (counter)
typedef unsigned long   DPSshandle;       // handle to system
typedef unsigned int	DPSbitfield;      // operacoes binarias
typedef void*           DPShandle;        // handle to particle/object/force

/*
 * Definition of particle states type
 */ 

typedef struct _DPSstate_ {
  DPSvector3f __DPS_STATE_POSITION           ;  //= {0.0f, 0.0f, 0.0f};
  DPSvector3f __DPS_STATE_VELOCITY           ;  //= {0.0f, 0.0f, 0.0f};
  DPSvector4f __DPS_STATE_COLOR              ;  //= {0.0f, 0.0f, 0.0f, 0.0f};
  DPSvector4f __DPS_STATE_COLOR_DEV          ;  //= {0.0f, 0.0f, 0.0f, 0.0f};
  DPSfloat    __DPS_STATE_AGE                ;  //= 0.0f;
  DPSfloat    __DPS_STATE_MASS               ;  //= 0.0f;
  DPSboolean  __DPS_STATE_ROLL               ;  //= DPS_FALSE;
  DPSfloat    __DPS_STATE_DT	             ;  //= 0.0f;   // Incremento de tempo

  DPSvoid     (*__DPS_STATE_MESH)(DPSulong)  ;  //= DPS_NULL;
  DPSvoid     (*__DPS_STATE_COLLIDE)
                (DPShandle, 
                DPSshandle, 
                DPSshandle, 
                DPSshandle)                  ;//= DPS_NULL; 
  DPSvoid     (*__DPS_STATE_DIE)
                (DPShandle, 
                DPSshandle, 
                DPSshandle, 
                DPSshandle)                  ;//= DPS_NULL; 


} DPSstate;


/*
 * Definition of tuple to keep 3 sets
 */ 

 
typedef struct _DPSsets_ {
  DPSshandle particles;
  DPSshandle forces; 
  DPSshandle objects; 
} DPSsets;



#endif
